@@ -544,7 +544,7 @@ bool GifFormat::onSave(FileOp* fop)
   int sprite_h = sprite->height();
   PixelFormat sprite_format = sprite->pixelFormat();
   bool interlaced = gif_options->interlaced();
-  int loop = 0;
+  int loop = (gif_options->loop() ? 0: -1);
   bool has_background = (sprite->backgroundLayer() ? true: false);
   int background_color = (sprite_format == IMAGE_INDEXED ? sprite->transparentColor(): 0);
   int transparent_index = (has_background ? -1: sprite->transparentColor());
@@ -707,20 +707,6 @@ bool GifFormat::onSave(FileOp* fop)
         throw Exception("Error writing GIF graphics extension record (trailer section).");
     }
 
-    // Add Aseprite block (at this moment, it's empty).
-    if (frame_num == 0) {
-      if (EGifPutExtensionLeader(gif_file, APPLICATION_EXT_FUNC_CODE) == GIF_ERROR)
-        throw Exception("Error writing GIF comment (header section).");
-
-      unsigned char extension_bytes[11];
-      memcpy(extension_bytes, "ASEPRITE1.0", 11);
-      if (EGifPutExtensionBlock(gif_file, sizeof(extension_bytes), extension_bytes) == GIF_ERROR)
-        throw Exception("Error writing GIF comment (first block).");
-
-      if (EGifPutExtensionTrailer(gif_file) == GIF_ERROR)
-        throw Exception("Error writing GIF comment (trailer section).");
-    }
-
     // Write graphics extension record (to save the duration of the
     // frame and maybe the transparency index).
     {
@@ -811,6 +797,7 @@ SharedPtr<FormatOptions> GifFormat::onGetFormatOptions(FileOp* fop)
     // Configuration parameters
     gif_options->setQuantize((GifOptions::Quantize)get_config_int("GIF", "Quantize", (int)gif_options->quantize()));
     gif_options->setInterlaced(get_config_bool("GIF", "Interlaced", gif_options->interlaced()));
+    gif_options->setLoop(get_config_bool("GIF", "Loop", gif_options->loop()));
     gif_options->setDithering((raster::DitheringMethod)get_config_int("GIF", "Dither", (int)gif_options->dithering()));
 
     // Load the window to ask to the user the GIF options he wants.
@@ -824,6 +811,7 @@ SharedPtr<FormatOptions> GifFormat::onGetFormatOptions(FileOp* fop)
       case GifOptions::QuantizeAll: win.quantizeAll()->setSelected(true); break;
     }
     win.interlaced()->setSelected(gif_options->interlaced());
+    win.loop()->setSelected(gif_options->loop());
 
     win.dither()->setEnabled(true);
     win.dither()->setSelected(gif_options->dithering() == raster::DITHERING_ORDERED);
@@ -839,12 +827,14 @@ SharedPtr<FormatOptions> GifFormat::onGetFormatOptions(FileOp* fop)
         gif_options->setQuantize(GifOptions::NoQuantize);
 
       gif_options->setInterlaced(win.interlaced()->isSelected());
+      gif_options->setLoop(win.loop()->isSelected());
       gif_options->setDithering(win.dither()->isSelected() ?
         raster::DITHERING_ORDERED:
         raster::DITHERING_NONE);
 
       set_config_int("GIF", "Quantize", gif_options->quantize());
       set_config_bool("GIF", "Interlaced", gif_options->interlaced());
+      set_config_bool("GIF", "Loop", gif_options->loop());
       set_config_int("GIF", "Dither", gif_options->dithering());
     }
     else {
