@@ -38,6 +38,7 @@
 #include "app/ui/skin/skin_theme.h"
 #include "app/ui_context.h"
 #include "base/bind.h"
+#include "base/scoped_value.h"
 #include "base/unique_ptr.h"
 #include "raster/brush.h"
 #include "raster/conversion_she.h"
@@ -63,6 +64,8 @@ using namespace gfx;
 using namespace ui;
 using namespace tools;
 
+static bool g_updatingFromTool = false;
+
 class ContextBar::BrushTypeField : public ButtonSet {
 public:
   BrushTypeField()
@@ -184,6 +187,8 @@ public:
 private:
   void onValueChange() override {
     IntEntry::onValueChange();
+    if (g_updatingFromTool)
+      return;
 
     ISettings* settings = UIContext::instance()->settings();
     Tool* currentTool = settings->getCurrentTool();
@@ -205,6 +210,8 @@ public:
 protected:
   void onValueChange() override {
     IntEntry::onValueChange();
+    if (g_updatingFromTool)
+      return;
 
     ISettings* settings = UIContext::instance()->settings();
     Tool* currentTool = settings->getCurrentTool();
@@ -230,6 +237,8 @@ public:
 protected:
   void onValueChange() override {
     IntEntry::onValueChange();
+    if (g_updatingFromTool)
+      return;
 
     ISettings* settings = UIContext::instance()->settings();
     Tool* currentTool = settings->getCurrentTool();
@@ -334,6 +343,8 @@ public:
 protected:
   void onValueChange() override {
     IntEntry::onValueChange();
+    if (g_updatingFromTool)
+      return;
 
     ISettings* settings = UIContext::instance()->settings();
     Tool* currentTool = settings->getCurrentTool();
@@ -351,6 +362,8 @@ public:
 protected:
   void onValueChange() override {
     IntEntry::onValueChange();
+    if (g_updatingFromTool)
+      return;
 
     ISettings* settings = UIContext::instance()->settings();
     Tool* currentTool = settings->getCurrentTool();
@@ -368,6 +381,8 @@ public:
 protected:
   void onValueChange() override {
     IntEntry::onValueChange();
+    if (g_updatingFromTool)
+      return;
 
     ISettings* settings = UIContext::instance()->settings();
     Tool* currentTool = settings->getCurrentTool();
@@ -706,6 +721,23 @@ protected:
   }
 };
 
+class ContextBar::AutoSelectLayerField : public CheckBox
+{
+public:
+  AutoSelectLayerField() : CheckBox("Auto Select Layer") {
+    setup_mini_font(this);
+  }
+
+protected:
+  void onClick(Event& ev) override {
+    CheckBox::onClick(ev);
+
+    UIContext::instance()->settings()->setAutoSelectLayer(isSelected());
+
+    releaseFocus();
+  }
+};
+
 ContextBar::ContextBar()
   : Box(JI_HORIZONTAL)
   , m_toolSettings(NULL)
@@ -736,6 +768,8 @@ ContextBar::ContextBar()
 
   addChild(m_grabAlpha = new GrabAlphaField());
 
+  addChild(m_autoSelectLayer = new AutoSelectLayerField());
+
   // addChild(new InkChannelTargetField());
   // addChild(new InkShadeField());
   // addChild(new InkSelectionField());
@@ -841,6 +875,8 @@ void ContextBar::onDropPixels(ContextBarObserver::DropAction action)
 
 void ContextBar::updateFromTool(tools::Tool* tool)
 {
+  base::ScopedValue<bool> lockFlag(g_updatingFromTool, true, false);
+
   ISettings* settings = UIContext::instance()->settings();
   IToolSettings* toolSettings = settings->getToolSettings(tool);
   IBrushSettings* brushSettings = toolSettings->getBrush();
@@ -861,6 +897,7 @@ void ContextBar::updateFromTool(tools::Tool* tool)
   m_inkOpacity->setTextf("%d", toolSettings->getOpacity());
 
   m_grabAlpha->setSelected(settings->getGrabAlpha());
+  m_autoSelectLayer->setSelected(settings->getAutoSelectLayer());
   m_freehandAlgo->setFreehandAlgorithm(toolSettings->getFreehandAlgorithm());
 
   m_sprayWidth->setValue(toolSettings->getSprayWidth());
@@ -877,6 +914,11 @@ void ContextBar::updateFromTool(tools::Tool* tool)
     (tool->getInk(0)->isEyedropper() ||
      tool->getInk(1)->isEyedropper());
 
+  // True if the current tool is move tool.
+  bool isMove =
+    (tool->getInk(0)->isCelMovement() ||
+     tool->getInk(1)->isCelMovement());
+
   // True if it makes sense to change the ink property for the current
   // tool.
   bool hasInk = hasOpacity;
@@ -904,6 +946,7 @@ void ContextBar::updateFromTool(tools::Tool* tool)
   m_inkType->setVisible(hasInk);
   m_inkOpacity->setVisible(hasOpacity);
   m_grabAlpha->setVisible(isEyedropper);
+  m_autoSelectLayer->setVisible(isMove);
   m_freehandBox->setVisible(isFreehand && hasOpacity);
   m_toleranceLabel->setVisible(hasTolerance);
   m_tolerance->setVisible(hasTolerance);
@@ -929,7 +972,7 @@ void ContextBar::updateForMovingPixels()
   layout();
 }
 
-void ContextBar::updateForSelectionMode(SelectionMode mode)
+void ContextBar::updateSelectionMode(SelectionMode mode)
 {
   if (!m_selectionMode->isVisible())
     return;
@@ -937,4 +980,12 @@ void ContextBar::updateForSelectionMode(SelectionMode mode)
   m_selectionMode->setSelectionMode(mode);
 }
 
+void ContextBar::updateAutoSelectLayer(bool state)
+{
+  if (!m_autoSelectLayer->isVisible())
+    return;
+
+  m_autoSelectLayer->setSelected(state);
+}
+
 } // namespace app
