@@ -137,6 +137,8 @@ private:
   Graphics* m_g;
 };
 
+static raster::ImageBufferPtr render_buffer;
+
 Editor::Editor(Document* document, EditorFlags flags)
   : Widget(editor_type())
   , m_state(new StandbyState())
@@ -392,12 +394,16 @@ void Editor::drawOneSpriteUnclippedRect(ui::Graphics* g, const gfx::Rect& rc, in
     RenderEngine renderEngine(m_document, m_sprite, m_layer, m_frame);
 
     // Generate the rendered image
+    if (!render_buffer)
+      render_buffer.reset(new raster::ImageBuffer());
+
     base::UniquePtr<Image> rendered(NULL);
     try {
       rendered.reset(renderEngine.renderSprite(
           source_x, source_y, width, height,
           m_frame, m_zoom, true,
-          ((m_flags & kShowOnionskin) == kShowOnionskin)));
+          ((m_flags & kShowOnionskin) == kShowOnionskin),
+          render_buffer));
     }
     catch (const std::exception& e) {
       Console::showException(e);
@@ -412,9 +418,11 @@ void Editor::drawOneSpriteUnclippedRect(ui::Graphics* g, const gfx::Rect& rc, in
       }
 
       she::Surface* tmp(she::instance()->createRgbaSurface(width, height));
-      convert_image_to_surface(rendered, m_sprite->getPalette(m_frame),
-        tmp, 0, 0, 0, 0, width, height);
-      g->blit(tmp, 0, 0, dest_x, dest_y, width, height);
+      if (tmp->nativeHandle()) {
+        convert_image_to_surface(rendered, m_sprite->getPalette(m_frame),
+          tmp, 0, 0, 0, 0, width, height);
+        g->blit(tmp, 0, 0, dest_x, dest_y, width, height);
+      }
       tmp->dispose();
     }
   }
@@ -674,39 +682,31 @@ void Editor::drawGrid(Graphics* g, const gfx::Rect& spriteBounds, const Rect& gr
 
 void Editor::flashCurrentLayer()
 {
-#if 0                           // TODO this flash effect can be done
-                                // only with hardware acceleration.
-                                // Finish it when the
-                                // Allegro 5 port is ready.
+  if (!UIContext::instance()->settings()->experimental()->flashLayer())
+    return;
+
+  DocumentLocation loc = getDocumentLocation();
+
   int x, y;
-  const Image* src_image = m_sprite->getCurrentImage(&x, &y);
+  const Image* src_image = loc.image(&x, &y);
   if (src_image) {
+    RenderEngine::setPreviewImage(NULL, FrameNumber(0), NULL);
+
     m_document->prepareExtraCel(0, 0, m_sprite->width(), m_sprite->height(), 255);
     Image* flash_image = m_document->getExtraCelImage();
-    int u, v;
 
-    clear_image(flash_image, flash_image->mask_color);
-    for (v=0; v<flash_image->height(); ++v) {
-      for (u=0; u<flash_image->width(); ++u) {
-        if (u-x >= 0 && u-x < src_image->width() &&
-            v-y >= 0 && v-y < src_image->height()) {
-          uint32_t color = get_pixel(src_image, u-x, v-y);
-          if (color != src_image->mask_color) {
-            Color ccc = Color::fromRgb(255, 255, 255);
-            put_pixel(flash_image, u, v,
-                      color_utils::color_for_image(ccc, flash_image->imgtype));
-          }
-        }
-      }
-    }
+    clear_image(flash_image, flash_image->maskColor());
+    copy_image(flash_image, src_image, x, y);
+    m_document->setExtraCelBlendMode(BLEND_MODE_BLACKANDWHITE);
 
-    drawSpriteSafe(0, 0, m_sprite->width()-1, m_sprite->height()-1);
-    gui_flip_screen();
+    drawSpriteClipped(gfx::Region(
+        gfx::Rect(0, 0, m_sprite->width(), m_sprite->height())));
+    gui_feedback();
 
-    clear_image(flash_image, flash_image->mask_color);
-    drawSpriteSafe(0, 0, m_sprite->width()-1, m_sprite->height()-1);
+    m_document->setExtraCelBlendMode(BLEND_MODE_NORMAL);
+    m_document->destroyExtraCel();
+    invalidate();
   }
-#endif
 }
 
 gfx::Point Editor::autoScroll(MouseMessage* msg, AutoScroll dir, bool blit_valid_rgn)
@@ -738,7 +738,7 @@ gfx::Point Editor::autoScroll(MouseMessage* msg, AutoScroll dir, bool blit_valid
     }
     setEditorScroll(scroll.x, scroll.y, blit_valid_rgn);
 
-#ifdef WIN32
+#if defined(WIN32) || defined(__APPLE__)
     mousePos -= delta;
     ui::set_mouse_position(mousePos);
 #endif
@@ -976,7 +976,8 @@ Rect Editor::getVisibleSpriteBounds()
   screenToEditor(vp.x, vp.y, &x1, &y1);
   screenToEditor(vp.x+vp.w-1, vp.y+vp.h-1, &x2, &y2);
 
-  return Rect(x1, y1, x2-x1+1, y2-y1+1);
+  return Rect(0, 0, m_sprite->width(), m_sprite->height())
+    .createIntersect(Rect(x1, y1, x2-x1+1, y2-y1+1));
 }
 
 // Changes the scroll to see the given point as the center of the editor.
@@ -1018,11 +1019,34 @@ void Editor::updateQuicktool()
     }
 
     tools::Tool* old_quicktool = m_quicktool;
-    m_quicktool = m_customizationDelegate->getQuickTool(current_tool);
+    tools::Tool* new_quicktool = m_customizationDelegate->getQuickTool(current_tool);
+
+    // Check if the current state accept the given quicktool.
+    if (new_quicktool && !m_state->acceptQuickTool(new_quicktool))
+      return;
+
+    // Hide the drawing cursor with the current tool brush size before
+    // we change the quicktool. In this way we avoid using the
+    // quicktool brush size to clean the current tool cursor.
+    //
+    // TODO Remove EditorState::regenerateDrawingCursor() creating a
+    // new Document concept of multiple extra cels: we need an extra
+    // cel for the drawing cursor, other for the moving pixels,
+    // etc. In this way we'll not have conflicts between different
+    // uses of the same extra cel.
+    if (m_state->regenerateDrawingCursor())
+      hideDrawingCursor();
+
+    m_quicktool = new_quicktool;
+
+    if (m_state->regenerateDrawingCursor())
+      showDrawingCursor();
 
     // If the tool has changed, we must to update the status bar because
     // the new tool can display something different in the status bar (e.g. Eyedropper)
     if (old_quicktool != m_quicktool) {
+      m_state->onQuickToolChange(this);
+
       updateStatusBar();
 
       App::instance()->getMainWindow()->getContextBar()
@@ -1031,12 +1055,16 @@ void Editor::updateQuicktool()
   }
 }
 
-void Editor::updateSelectionMode()
+void Editor::updateContextBarFromModifiers()
 {
   // We update the selection mode only if we're not selecting.
   if (hasCapture())
     return;
 
+  ContextBar* ctxBar = App::instance()->getMainWindow()->getContextBar();
+
+  // Selection mode
+
   SelectionMode mode = UIContext::instance()->settings()->selection()->getSelectionMode();
 
   if (m_customizationDelegate && m_customizationDelegate->isAddSelectionPressed())
@@ -1048,9 +1076,19 @@ void Editor::updateSelectionMode()
 
   if (mode != m_selectionMode) {
     m_selectionMode = mode;
+    ctxBar->updateSelectionMode(mode);
+  }
 
-    App::instance()->getMainWindow()->getContextBar()
-      ->updateForSelectionMode(mode);
+  // Move tool options
+
+  bool autoSelectLayer = UIContext::instance()->settings()->getAutoSelectLayer();
+
+  if (m_customizationDelegate && m_customizationDelegate->isAutoSelectLayerPressed())
+    autoSelectLayer = true;
+
+  if (m_autoSelectLayer != autoSelectLayer) {
+    m_autoSelectLayer = autoSelectLayer;
+    ctxBar->updateAutoSelectLayer(autoSelectLayer);
   }
 }
 
@@ -1080,7 +1118,7 @@ bool Editor::onProcessMessage(Message* msg)
 
     case kMouseEnterMessage:
       updateQuicktool();
-      updateSelectionMode();
+      updateContextBarFromModifiers();
       break;
 
     case kMouseLeaveMessage:
@@ -1097,7 +1135,7 @@ bool Editor::onProcessMessage(Message* msg)
           m_secondaryButton = mouseMsg->right();
 
           updateQuicktool();
-          updateSelectionMode();
+          updateContextBarFromModifiers();
           editor_setcursor();
         }
 
@@ -1122,7 +1160,7 @@ bool Editor::onProcessMessage(Message* msg)
           m_secondaryButton = false;
 
           updateQuicktool();
-          updateSelectionMode();
+          updateContextBarFromModifiers();
           editor_setcursor();
         }
 
@@ -1138,7 +1176,7 @@ bool Editor::onProcessMessage(Message* msg)
 
         if (hasMouse()) {
           updateQuicktool();
-          updateSelectionMode();
+          updateContextBarFromModifiers();
           editor_setcursor();
         }
 
@@ -1154,7 +1192,7 @@ bool Editor::onProcessMessage(Message* msg)
 
         if (hasMouse()) {
           updateQuicktool();
-          updateSelectionMode();
+          updateContextBarFromModifiers();
           editor_setcursor();
         }
 
@@ -1306,7 +1344,7 @@ void Editor::setZoomAndCenterInMouse(int zoom, int mouse_x, int mouse_y, ZoomBeh
 
   switch (zoomBehavior) {
     case kCofiguredZoomBehavior:
-      centerMouse = get_config_bool("Editor", "CenterMouseInZoom", true);
+      centerMouse = UIContext::instance()->settings()->getCenterOnZoom();
       break;
     case kCenterOnZoom:
       centerMouse = true;
@@ -1364,26 +1402,14 @@ void Editor::pasteImage(const Image* image, int x, int y)
 
   // Check bounds where the image will be pasted.
   {
-    // First we limit the image inside the sprite's bounds.
-    x = MID(0, x, sprite->width() - image->width());
-    y = MID(0, y, sprite->height() - image->height());
-
     // Then we check if the image will be visible by the user.
-    Rect visibleBounds = getVisibleSpriteBounds();
+    Rect visibleBounds = getVisibleSpriteBounds().shrink(4*jguiscale());
     x = MID(visibleBounds.x-image->width(), x, visibleBounds.x+visibleBounds.w-1);
     y = MID(visibleBounds.y-image->height(), y, visibleBounds.y+visibleBounds.h-1);
 
-    // If the visible part of the pasted image will not fit in the
-    // visible bounds of the editor, we put the image in the center of
-    // the visible bounds.
-    Rect visiblePasted = visibleBounds.createIntersect(gfx::Rect(x, y, image->width(), image->height()));
-    if (((visibleBounds.w >= image->width() && visiblePasted.w < image->width()/2) ||
-         (visibleBounds.w <  image->width() && visiblePasted.w < visibleBounds.w/2)) ||
-        ((visibleBounds.h >= image->height() && visiblePasted.h < image->width()/2) ||
-         (visibleBounds.h <  image->height() && visiblePasted.h < visibleBounds.h/2))) {
-      x = visibleBounds.x + visibleBounds.w/2 - image->width()/2;
-      y = visibleBounds.y + visibleBounds.h/2 - image->height()/2;
-    }
+    // We limit the image inside the sprite's bounds.
+    x = MID(0, x, sprite->width() - image->width());
+    y = MID(0, y, sprite->height() - image->height());
   }
 
   PixelsMovementPtr pixelsMovement(
@@ -1412,6 +1438,12 @@ void Editor::notifyScrollChanged()
   m_observers.notifyScrollChanged(this);
 }
 
+// static
+ImageBufferPtr Editor::getRenderImageBuffer()
+{
+  return render_buffer;
+}
+
 void Editor::onSetTiledMode(filters::TiledMode mode)
 {
   invalidate();
