@@ -122,6 +122,11 @@ void StandbyState::onCurrentToolChange(Editor* editor)
   editor->invalidate();
 }
 
+void StandbyState::onQuickToolChange(Editor* editor)
+{
+  editor->invalidate();
+}
+
 bool StandbyState::checkForScroll(Editor* editor, MouseMessage* msg)
 {
   tools::Ink* clickedInk = editor->getCurrentEditorInk();
@@ -160,7 +165,6 @@ bool StandbyState::onMouseDown(Editor* editor, MouseMessage* msg)
     return true;
 
   UIContext* context = UIContext::instance();
-  tools::Tool* currentTool = editor->getCurrentEditorTool();
   tools::Ink* clickedInk = editor->getCurrentEditorInk();
   DocumentLocation location;
   editor->getDocumentLocation(&location);
@@ -176,71 +180,97 @@ bool StandbyState::onMouseDown(Editor* editor, MouseMessage* msg)
 
   // Move cel X,Y coordinates
   if (clickedInk->isCelMovement()) {
+    // Handle "Auto Select Layer"
+    if (editor->isAutoSelectLayer()) {
+      ColorPicker picker;
+      int x, y;
+
+      editor->screenToEditor(
+        msg->position().x,
+        msg->position().y, &x, &y);
+
+      picker.pickColor(location, x, y, ColorPicker::FromComposition);
+
+      if (layer != picker.layer()) {
+        layer = picker.layer();
+        if (layer) {
+          editor->setLayer(layer);
+          editor->flashCurrentLayer();
+        }
+      }
+    }
+
     if ((layer) &&
         (layer->type() == OBJECT_LAYER_IMAGE)) {
-      // TODO you can move the `Background' with tiled mode
+      // TODO we should be able to move the `Background' with tiled mode
       if (layer->isBackground()) {
-        Alert::show(PACKAGE
-                    "<<You can't move the `Background' layer."
-                    "||&Close");
+        StatusBar::instance()->showTip(1000,
+          "The background layer cannot be moved");
       }
-      else if (!layer->isMoveable()) {
-        Alert::show(PACKAGE "<<The layer movement is locked.||&Close");
+      else if (!layer->isReadable()) {
+        StatusBar::instance()->showTip(1000,
+          "Layer '%s' is hidden", layer->name().c_str());
+      }
+      else if (!layer->isMoveable() || !layer->isWritable()) {
+        StatusBar::instance()->showTip(1000,
+          "Layer '%s' is locked", layer->name().c_str());
       }
       else {
         // Change to MovingCelState
         editor->setState(EditorStatePtr(new MovingCelState(editor, msg)));
       }
     }
-    return true;
-  }
 
-  // Transform selected pixels
-  if (document->isMaskVisible() &&
-      m_decorator->getTransformHandles(editor)) {
-    TransformHandles* transfHandles = m_decorator->getTransformHandles(editor);
-
-    // Get the handle covered by the mouse.
-    HandleType handle = transfHandles->getHandleAtPoint(editor,
-                                                        msg->position(),
-                                                        document->getTransformation());
-
-    if (handle != NoHandle) {
-      int x, y, opacity;
-      Image* image = location.image(&x, &y, &opacity);
-      if (image) {
-        if (!layer->isWritable()) {
-          Alert::show(PACKAGE "<<The layer is locked.||&Close");
-          return true;
-        }
-
-        // Change to MovingPixelsState
-        transformSelection(editor, msg, handle);
-      }
-      return true;
-    }
-  }
-
-  // Move selected pixels
-  if (editor->isInsideSelection() &&
-      currentTool->getInk(0)->isSelection() &&
-      msg->left()) {
-    if (!layer->isWritable()) {
-      Alert::show(PACKAGE "<<The layer is locked.||&Close");
-      return true;
-    }
-
-    // Change to MovingPixelsState
-    transformSelection(editor, msg, MoveHandle);
     return true;
   }
 
   // Call the eyedropper command
   if (clickedInk->isEyedropper()) {
-    onMouseMove(editor, msg);
+    callEyedropper(editor);
     return true;
   }
 
+  if (clickedInk->isSelection()) {
+    // Transform selected pixels
+    if (document->isMaskVisible() && m_decorator->getTransformHandles(editor)) {
+      TransformHandles* transfHandles = m_decorator->getTransformHandles(editor);
+
+      // Get the handle covered by the mouse.
+      HandleType handle = transfHandles->getHandleAtPoint(editor,
+        msg->position(),
+        document->getTransformation());
+
+      if (handle != NoHandle) {
+        int x, y, opacity;
+        Image* image = location.image(&x, &y, &opacity);
+        if (layer && image) {
+          if (!layer->isWritable()) {
+            StatusBar::instance()->showTip(1000,
+              "Layer '%s' is locked", layer->name().c_str());
+            return true;
+          }
+
+          // Change to MovingPixelsState
+          transformSelection(editor, msg, handle);
+        }
+        return true;
+      }
+    }
+
+    // Move selected pixels
+    if (layer && editor->isInsideSelection() && msg->left()) {
+      if (!layer->isWritable()) {
+        StatusBar::instance()->showTip(1000,
+          "Layer '%s' is locked", layer->name().c_str());
+        return true;
+      }
+
+      // Change to MovingPixelsState
+      transformSelection(editor, msg, MoveHandle);
+      return true;
+    }
+  }
+
   // Start the Tool-Loop
   if (layer) {
     tools::ToolLoop* toolLoop = create_tool_loop(editor, context);
@@ -263,16 +293,8 @@ bool StandbyState::onMouseMove(Editor* editor, MouseMessage* msg)
   // We control eyedropper tool from here. TODO move this to another place
   if (msg->left() || msg->right()) {
     tools::Ink* clickedInk = editor->getCurrentEditorInk();
-    if (clickedInk->isEyedropper()) {
-      Command* eyedropper_cmd =
-        CommandsModule::instance()->getCommandByName(CommandId::Eyedropper);
-      bool fg = (static_cast<tools::PickInk*>(clickedInk)->target() == tools::PickInk::Fg);
-
-      Params params;
-      params.set("target", fg ? "foreground": "background");
-
-      UIContext::instance()->executeCommand(eyedropper_cmd, &params);
-    }
+    if (clickedInk->isEyedropper())
+      callEyedropper(editor);
   }
 
   editor->moveDrawingCursor();
@@ -556,9 +578,27 @@ void StandbyState::transformSelection(Editor* editor, MouseMessage* msg, HandleT
     // Other editor is locking the document.
 
     // TODO steal the PixelsMovement of the other editor and use it for this one.
+    StatusBar::instance()->showTip(1000, "The sprite is locked in other editor");
+    jmouse_set_cursor(kForbiddenCursor);
   }
 }
 
+void StandbyState::callEyedropper(Editor* editor)
+{
+  tools::Ink* clickedInk = editor->getCurrentEditorInk();
+  if (!clickedInk->isEyedropper())
+    return;
+
+  Command* eyedropper_cmd =
+    CommandsModule::instance()->getCommandByName(CommandId::Eyedropper);
+  bool fg = (static_cast<tools::PickInk*>(clickedInk)->target() == tools::PickInk::Fg);
+
+  Params params;
+  params.set("target", fg ? "foreground": "background");
+
+  UIContext::instance()->executeCommand(eyedropper_cmd, &params);
+}
+
 //////////////////////////////////////////////////////////////////////
 // Decorator
 
