@@ -22,6 +22,8 @@
 
 #include "app/modules/gui.h"
 #include "app/resource_finder.h"
+#include "app/ui/app_menuitem.h"
+#include "app/ui/keyboard_shortcuts.h"
 #include "app/ui/skin/button_icon_impl.h"
 #include "app/ui/skin/skin_property.h"
 #include "app/ui/skin/skin_slider_property.h"
@@ -1324,17 +1326,19 @@ void SkinTheme::paintMenuItem(ui::PaintEvent& ev)
       }
     }
     // Draw the keyboard shortcut
-    else if (widget->getAccel()) {
-      int old_align = widget->getAlign();
+    else if (AppMenuItem* appMenuItem = dynamic_cast<AppMenuItem*>(widget)) {
+      if (appMenuItem->getKey() && !appMenuItem->getKey()->accels().empty()) {
+        int old_align = appMenuItem->getAlign();
 
-      pos = bounds;
-      pos.w -= widget->child_spacing/4;
+        pos = bounds;
+        pos.w -= widget->child_spacing/4;
 
-      std::string buf = widget->getAccel()->toString();
+        std::string buf = appMenuItem->getKey()->accels().front().toString();
 
-      widget->setAlign(JI_RIGHT | JI_MIDDLE);
-      drawTextString(g, buf.c_str(), fg, ColorNone, widget, pos, 0);
-      widget->setAlign(old_align);
+        widget->setAlign(JI_RIGHT | JI_MIDDLE);
+        drawTextString(g, buf.c_str(), fg, ColorNone, widget, pos, 0);
+        widget->setAlign(old_align);
+      }
     }
   }
 }
